@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2007-2015 Douglas Gilbert.
+ * Copyright (c) 2007-2016 Douglas Gilbert.
  * All rights reserved.
  * Use of this source code is governed by a BSD-style
  * license that can be found in the BSD_LICENSE file.
@@ -17,7 +17,7 @@
 #endif
 
 
-const char * sg_lib_version_str = "2.14 20150429";  /* spc5r02, sbc4r02 */
+const char * sg_lib_version_str = "2.17 20160202";  /* spc5r08, sbc4r10 */
 
 
 /* indexed by pdt; those that map to own index do not decay */
@@ -107,7 +107,7 @@ struct sg_lib_value_name_t sg_lib_normal_opcodes[] = {
     {0x3b, 0, "Write buffer"},
     {0x3c, 0, "Read buffer(10)"},
     {0x3d, 0, "Update block"},
-    {0x3e, 0, "Read long(10)"},  /* SBC-3 r31 recommends Read long(16) */
+    {0x3e, 0, "Read long(10)"},         /* obsolete in SBC-4 r7 */
     {0x3f, 0, "Write long(10)"}, /* SBC-3 r31 recommends Write long(16) */
     {0x40, 0, "Change definition"},     /* obsolete in SPC-4 r11 */
     {0x41, 0, "Write same(10)"}, /* SBC-3 r31 recommends Write same(16) */
@@ -172,6 +172,7 @@ struct sg_lib_value_name_t sg_lib_normal_opcodes[] = {
     {0x93, PDT_TAPE, "Erase(16)"},
     {0x94, PDT_ZBC, "ZBC out"},  /* new sbc4r04, has service actions */
     {0x95, PDT_ZBC, "ZBC in"},   /* new sbc4r04, has service actions */
+    {0x9a, 0, "Write stream(16)"},      /* added sbc4r07 */
     {0x9b, 0, "Read buffer(16)"},       /* added spc5r02 */
     {0x9c, 0, "Write atomic(16)"},
     {0x9d, 0, "Service action bidirectional"},  /* added spc4r35 */
@@ -259,6 +260,15 @@ struct sg_lib_value_name_t sg_lib_write_buff_arr[] = {
     {0xffff, 0, NULL},
 };
 
+/* Read position (SSC) [0x34] service actions, need prefix */
+struct sg_lib_value_name_t sg_lib_read_pos_arr[] = {
+    {0x0, PDT_TAPE, "short form - block id"},
+    {0x1, PDT_TAPE, "short form - vendor specific"},
+    {0x6, PDT_TAPE, "long form"},
+    {0x8, PDT_TAPE, "extended form"},
+    {0xffff, 0, NULL},
+};
+
 /* Maintenance in [0xa3] service actions */
 struct sg_lib_value_name_t sg_lib_maint_in_arr[] = {
     {0x5, 0, "Report identifying information"},
@@ -270,6 +280,7 @@ struct sg_lib_value_name_t sg_lib_maint_in_arr[] = {
     {0xe, 0, "Report priority"},
     {0xf, 0, "Report timestamp"},
     {0x10, 0, "Management protocol in"},
+    {0x1d, 0, "Report provisioning initialization pattern"},
     {0x1f, 0, "Maintenance in vendor specific"},
     {0xffff, 0, NULL},
 };
@@ -312,10 +323,12 @@ struct sg_lib_value_name_t sg_lib_serv_out12_arr[] = {
 /* Service action in(16) [0x9e] service actions */
 struct sg_lib_value_name_t sg_lib_serv_in16_arr[] = {
     {0x10, 0, "Read capacity(16)"},
-    {0x11, 0, "Read long(16)"},
+    {0x11, 0, "Read long(16)"},         /* obsolete in SBC-4 r7 */
     {0x12, 0, "Get LBA status"},
     {0x13, 0, "Report referrals"},
-    {0x14, PDT_ZBC, "Report zones"},
+    {0x14, 0, "Stream control"},
+    {0x15, 0, "Background control"},
+    {0x16, 0, "Get stream status"},
     {0xffff, 0, NULL},
 };
 
@@ -399,6 +412,7 @@ struct sg_lib_value_name_t sg_lib_variable_length_arr[] = {
     {0xd, 0, "Write same(32)"},
     {0xe, 0, "Orwrite(32)"},         /* added sbc3r25 */
     {0xf, 0, "Atomic write(32)"},    /* added sbc4r02 */
+    {0x10, 0, "Write stream(32)"},   /* added sbc4r07 */
     {0x1800, 0, "Receive credential"},
     {0x8801, 0, "Format OSD (osd)"},
     {0x8802, 0, "Create (osd)"},
@@ -472,6 +486,16 @@ struct sg_lib_value_name_t sg_lib_zoning_in_arr[] = {
     {0xffff, 0, NULL},
 };
 
+/* Read attribute [0x8c] service actions */
+struct sg_lib_value_name_t sg_lib_read_attr_arr[] = {
+    {0x0, 0, "attribute values"},
+    {0x1, 0, "attribute list"},
+    {0x2, 0, "logical volume list"},
+    {0x3, 0, "partition list"},
+    {0x5, 0, "supported attributes"},
+    {0xffff, 0, NULL},
+};
+
 #else   /* SG_SCSI_STRINGS */
 
 struct sg_lib_value_name_t sg_lib_normal_opcodes[] = {
@@ -546,6 +570,10 @@ struct sg_lib_value_name_t sg_lib_zoning_in_arr[] = {
     {0xffff, 0, NULL},
 };
 
+struct sg_lib_value_name_t sg_lib_read_attr_arr[] = {
+    {0xffff, 0, NULL},
+};
+
 #endif  /* SG_SCSI_STRINGS */
 
 /* A conveniently formatted list of SCSI ASC/ASCQ codes and their
@@ -791,6 +819,9 @@ struct sg_lib_asc_ascq_t sg_lib_asc_ascq[] =
     {0x20,0x0A,"Access denied - invalid proxy token"},
     {0x20,0x0B,"Access denied - ACL LUN conflict"},
     {0x20,0x0C,"Illegal command when not in append-only mode"},
+    {0x20,0x0D,"Not an administrative logical unit"},
+    {0x20,0x0E,"Not a subsidiary logical unit"},
+    {0x20,0x0F,"Not a conglomerate logical unit"},
     {0x21,0x00,"Logical block address out of range"},
     {0x21,0x01,"Invalid element address"},
     {0x21,0x02,"Invalid address for write"},
@@ -799,6 +830,7 @@ struct sg_lib_asc_ascq_t sg_lib_asc_ascq[] =
     {0x21,0x05,"Write boundary violation"},
     {0x21,0x06,"Attempt to read invalid data"},
     {0x21,0x07,"Read boundary violation"},
+    {0x21,0x08,"Misaligned write command"},
     {0x22,0x00,"Illegal function (use 20 00, 24 00, or 26 00)"},
     {0x23,0x00,"Invalid token operation, cause not reportable"},
     {0x23,0x01,"Invalid token operation, unsupported token type"},
@@ -883,6 +915,7 @@ struct sg_lib_asc_ascq_t sg_lib_asc_ascq[] =
     {0x2A,0x0b,"Error history snapshot released"},
     {0x2A,0x14,"SA creation capabilities data has changed"},
     {0x2A,0x15,"Medium removal prevention preempted"},
+    {0x2A,0x16,"Zone reset write pointer recommended"},
     {0x2B,0x00,"Copy cannot execute since host cannot disconnect"},
     {0x2C,0x00,"Command sequence error"},
     {0x2C,0x01,"Too many windows specified"},
@@ -899,6 +932,9 @@ struct sg_lib_asc_ascq_t sg_lib_asc_ascq[] =
     {0x2C,0x0C,"ORWRITE generation does not match"},
     {0x2C,0x0D,"Reset write pointer not allowed"},
     {0x2C,0x0E,"Zone is offline"},
+    {0x2C,0x0F,"Stream not open"},
+    {0x2C,0x10,"Unwritten data in zone"},
+    {0x2C,0x11,"Descriptor format sense data required"},
     {0x2D,0x00,"Overwrite error on update in place"},
     {0x2E,0x00,"Insufficient time for operation"},
     {0x2E,0x01,"Command timeout before processing"},
@@ -1011,6 +1047,9 @@ struct sg_lib_asc_ascq_t sg_lib_asc_ascq[] =
     {0x3F,0x15,"Inspect referrals sense descriptors"},
     {0x3F,0x16,"Microcode has been changed without reset"},
     {0x3F,0x17,"Zone transition to full"},
+    {0x3F,0x18,"Bind completed"},
+    {0x3F,0x19,"Bind redirected"},
+    {0x3F,0x1A,"Subsidiary binding changed"},
 
     /*
      * ASC 0x40, 0x41 and 0x42 overridden by "additional2" array entries
@@ -1105,6 +1144,8 @@ struct sg_lib_asc_ascq_t sg_lib_asc_ascq[] =
     {0x55,0x0d,"Insufficient resources to create rod token"},
     {0x55,0x0e,"Insufficient zone resources"},
     {0x55,0x0f,"Insufficient zone resources to complete write"},
+    {0x55,0x10,"Maximum number of streams open"},
+    {0x55,0x11,"Insufficient resources to bind"},
     {0x57,0x00,"Unable to recover table-of-contents"},
     {0x58,0x00,"Generation does not exist"},
     {0x59,0x00,"Updated block read"},
@@ -1244,6 +1285,8 @@ struct sg_lib_asc_ascq_t sg_lib_asc_ascq[] =
     {0x67,0x09,"Multiply assigned logical unit"},
     {0x67,0x0A,"Set target port groups command failed"},
     {0x67,0x0B,"ATA device feature not enabled"},
+    {0x67,0x0C,"Command rejected"},
+    {0x67,0x0D,"Explicit bind not allowed"},
     {0x68,0x00,"Logical unit not configured"},
     {0x68,0x01,"Subsidiary logical unit not configured"},
     {0x69,0x00,"Data loss on logical unit"},
@@ -1265,6 +1308,9 @@ struct sg_lib_asc_ascq_t sg_lib_asc_ascq[] =
     {0x6F,0x05,"Drive region must be permanent/region reset count error"},
     {0x6F,0x06,"Insufficient block count for binding nonce recording"},
     {0x6F,0x07,"Conflict in binding nonce recording"},
+    {0x6F,0x08,"Insufficient permission"},
+    {0x6F,0x09,"Invalid drive-host pairing server"},
+    {0x6F,0x0A,"Drive-host pairing suspended"},
     /*
      * ASC 0x70 overridden by an "additional2" array entry
      * so there is no need to have them here.
@@ -1395,12 +1441,12 @@ const char * sg_lib_transport_proto_strs[] =
     "Serial Bus Protocol for IEEE 1394 (SBP-3)",
     "SCSI RDMA Protocol (SRP)",
     "Internet SCSI (iSCSI)",
-    "Serial Attached SCSI Protocol (SPL-3)",
+    "Serial Attached SCSI Protocol (SPL-4)",
     "Automation/Drive Interface Transport (ADT-2)",
     "AT Attachment Interface (ACS-2)",          /* 0x8 */
     "USB Attached SCSI (UAS-2)",
     "SCSI over PCI Express (SOP)",
-    "PCIe",                             /* added in spc5r<x> ?? */
+    "PCIe",                             /* added in spc5r02 */
     "Oxc", "Oxd", "Oxe",
     "No specific protocol"
 };
