@@ -23,10 +23,12 @@
 #include "sg_lib.h"
 #include "sg_cmds_basic.h"
 #include "sg_cmds_extra.h"
+#include "sg_unaligned.h"
+#include "sg_pr2serr.h"
 
 #include "sg_pt.h"
 
-static const char * version_str = "0.43 20150511";    /* spc4r37 */
+static const char * version_str = "0.45 20151219";    /* spc5r07 */
 
 
 #define SENSE_BUFF_LEN 64       /* Arbitrary, could be larger */
@@ -96,8 +98,7 @@ struct opts_t {
 static void
 usage()
 {
-    fprintf(stderr,
-            "Usage:  sg_opcodes [--alpha] [--compact] [--help] [--hex] "
+    pr2serr("Usage:  sg_opcodes [--alpha] [--compact] [--help] [--hex] "
             "[--mask]\n"
             "                   [--no-inquiry] [--opcode=OP[,SA]] [--raw] "
             "[--rctd]\n"
@@ -140,8 +141,7 @@ usage()
 static void
 usage_old()
 {
-    fprintf(stderr,
-            "Usage:  sg_opcodes [-a] [-c] [-H] [-m] [-n] [-o=OP] [-q] [-r] "
+    pr2serr("Usage:  sg_opcodes [-a] [-c] [-H] [-m] [-n] [-o=OP] [-q] [-r] "
             "[-R] [-s=SA]\n"
             "                   [-t] [-u] [-v] [-V] DEVICE\n"
             "  where:\n"
@@ -205,7 +205,7 @@ process_cl_new(struct opts_t * optsp, int argc, char * argv[])
             break;      /* ignore */
         case 'o':
             if (strlen(optarg) >= (sizeof(b) - 1)) {
-                fprintf(stderr, "argument to '--opcode' too long\n");
+                pr2serr("argument to '--opcode' too long\n");
                 return SG_LIB_SYNTAX_ERROR;
             }
             cp = strchr(optarg, ',');
@@ -214,13 +214,13 @@ process_cl_new(struct opts_t * optsp, int argc, char * argv[])
                 strncpy(b, optarg, cp - optarg);
                 n = sg_get_num(b);
                 if ((n < 0) || (n > 255)) {
-                    fprintf(stderr, "bad OP argument to '--opcode'\n");
+                    pr2serr("bad OP argument to '--opcode'\n");
                     return SG_LIB_SYNTAX_ERROR;
                 }
                 optsp->do_opcode = n;
                 n = sg_get_num(cp + 1);
                 if ((n < 0) || (n > 0xffff)) {
-                    fprintf(stderr, "bad SA argument to '--opcode'\n");
+                    pr2serr("bad SA argument to '--opcode'\n");
                     usage();
                     return SG_LIB_SYNTAX_ERROR;
                 }
@@ -228,7 +228,7 @@ process_cl_new(struct opts_t * optsp, int argc, char * argv[])
             } else {
                 n = sg_get_num(optarg);
                 if ((n < 0) || (n > 255)) {
-                    fprintf(stderr, "bad argument to '--opcode'\n");
+                    pr2serr("bad argument to '--opcode'\n");
                     usage();
                     return SG_LIB_SYNTAX_ERROR;
                 }
@@ -250,7 +250,7 @@ process_cl_new(struct opts_t * optsp, int argc, char * argv[])
         case 's':
             n = sg_get_num(optarg);
             if ((n < 0) || (n > 0xffff)) {
-                fprintf(stderr, "bad argument to '--sa'\n");
+                pr2serr("bad argument to '--sa'\n");
                 usage();
                 return SG_LIB_SYNTAX_ERROR;
             }
@@ -269,7 +269,7 @@ process_cl_new(struct opts_t * optsp, int argc, char * argv[])
             ++optsp->do_version;
             break;
         default:
-            fprintf(stderr, "unrecognised option code %c [0x%x]\n", c, c);
+            pr2serr("unrecognised option code %c [0x%x]\n", c, c);
             if (optsp->do_help)
                 break;
             usage();
@@ -283,8 +283,7 @@ process_cl_new(struct opts_t * optsp, int argc, char * argv[])
         }
         if (optind < argc) {
             for (; optind < argc; ++optind)
-                fprintf(stderr, "Unexpected extra argument: %s\n",
-                        argv[optind]);
+                pr2serr("Unexpected extra argument: %s\n", argv[optind]);
             usage();
             return SG_LIB_SYNTAX_ERROR;
         }
@@ -360,7 +359,7 @@ process_cl_old(struct opts_t * optsp, int argc, char * argv[])
             if (0 == strncmp("o=", cp, 2)) {
                 num = sscanf(cp + 2, "%x", (unsigned int *)&n);
                 if ((1 != num) || (n > 255)) {
-                    fprintf(stderr, "Bad number after 'o=' option\n");
+                    pr2serr("Bad number after 'o=' option\n");
                     usage_old();
                     return SG_LIB_SYNTAX_ERROR;
                 }
@@ -368,7 +367,7 @@ process_cl_old(struct opts_t * optsp, int argc, char * argv[])
             } else if (0 == strncmp("s=", cp, 2)) {
                 num = sscanf(cp + 2, "%x", (unsigned int *)&n);
                 if (1 != num) {
-                    fprintf(stderr, "Bad number after 's=' option\n");
+                    pr2serr("Bad number after 's=' option\n");
                     usage_old();
                     return SG_LIB_SYNTAX_ERROR;
                 }
@@ -376,15 +375,15 @@ process_cl_old(struct opts_t * optsp, int argc, char * argv[])
             } else if (0 == strncmp("-old", cp, 4))
                 ;
             else if (jmp_out) {
-                fprintf(stderr, "Unrecognized option: %s\n", cp);
+                pr2serr("Unrecognized option: %s\n", cp);
                 usage_old();
                 return SG_LIB_SYNTAX_ERROR;
             }
         } else if (NULL == optsp->device_name)
             optsp->device_name = cp;
         else {
-            fprintf(stderr, "too many arguments, got: %s, not expecting: "
-                    "%s\n", optsp->device_name, cp);
+            pr2serr("too many arguments, got: %s, not expecting: %s\n",
+                    optsp->device_name, cp);
             usage_old();
             return SG_LIB_SYNTAX_ERROR;
         }
@@ -438,10 +437,10 @@ opcode_num_compare(const void * left, const void * right)
         return -1;
     l_opc = ll[0];
     if (ll[5] & 1)
-        l_serv_act = ((ll[2] << 8) | ll[3]);
+        l_serv_act = sg_get_unaligned_be16(ll + 2);
     r_opc = rr[0];
     if (rr[5] & 1)
-        r_serv_act = ((rr[2] << 8) | rr[3]);
+        r_serv_act = sg_get_unaligned_be16(rr + 2);
     if (l_opc < r_opc)
         return -1;
     if (l_opc > r_opc)
@@ -471,13 +470,13 @@ opcode_alpha_compare(const void * left, const void * right)
         return -1;
     l_opc = ll[0];
     if (ll[5] & 1)
-        l_serv_act = ((ll[2] << 8) | ll[3]);
+        l_serv_act = sg_get_unaligned_be16(ll + 2);
     l_name_buff[0] = '\0';
     sg_get_opcode_sa_name(l_opc, l_serv_act, peri_type,
                           NAME_BUFF_SZ, l_name_buff);
     r_opc = rr[0];
     if (rr[5] & 1)
-        r_serv_act = ((rr[2] << 8) | rr[3]);
+        r_serv_act = sg_get_unaligned_be16(rr + 2);
     r_name_buff[0] = '\0';
     sg_get_opcode_sa_name(r_opc, r_serv_act, peri_type,
                           NAME_BUFF_SZ, r_name_buff);
@@ -495,8 +494,7 @@ list_all_codes(unsigned char * rsoc_buff, int rsoc_len, struct opts_t * op,
     char sa_buff[8];
     unsigned char ** sort_arr = NULL;
 
-    cd_len = ((rsoc_buff[0] << 24) | (rsoc_buff[1] << 16) |
-              (rsoc_buff[2] << 8) | rsoc_buff[3]);
+    cd_len = sg_get_unaligned_be32(rsoc_buff + 0);
     if (cd_len > (rsoc_len - 4)) {
         printf("sg_opcodes: command data length=%d, allocation=%d; "
                "truncate\n", cd_len, rsoc_len - 4);
@@ -553,7 +551,7 @@ list_all_codes(unsigned char * rsoc_buff, int rsoc_len, struct opts_t * op,
         sa_v = ucp[5] & 1;
         serv_act = 0;
         if (sa_v) {
-            serv_act = ((ucp[2] << 8) | ucp[3]);
+            serv_act = sg_get_unaligned_be16(ucp + 2);
             sg_get_opcode_sa_name(opcode, serv_act, peri_type, NAME_BUFF_SZ,
                                   name_buff);
             if (op->do_compact)
@@ -571,15 +569,13 @@ list_all_codes(unsigned char * rsoc_buff, int rsoc_len, struct opts_t * op,
                            sa_buff);
                 else
                     printf(" %.2x     %.4s       %3d", opcode, sa_buff,
-                           ((ucp[6] << 8) | ucp[7]));
-                to = ((unsigned int)ucp[12] << 24) + (ucp[13] << 16) +
-                     (ucp[14] << 8) + ucp[15];
+                           sg_get_unaligned_be16(ucp + 6));
+                to = sg_get_unaligned_be32(ucp + 12);
                 if (0 == to)
                     printf("         -");
                 else
                     printf("  %8u", to);
-                to = ((unsigned int)ucp[16] << 24) + (ucp[17] << 16) +
-                     (ucp[18] << 8) + ucp[19];
+                to = sg_get_unaligned_be32(ucp + 16);
                 if (0 == to)
                     printf("          -");
                 else
@@ -591,15 +587,15 @@ list_all_codes(unsigned char * rsoc_buff, int rsoc_len, struct opts_t * op,
                            (sa_v ? ',' : ' '), sa_buff, name_buff);
                 else
                     printf(" %.2x     %.4s       %3d                         "
-                           "%s\n", opcode, sa_buff, ((ucp[6] << 8) | ucp[7]),
-                           name_buff);
+                           "%s\n", opcode, sa_buff,
+                           sg_get_unaligned_be16(ucp + 6), name_buff);
         } else
             if (op->do_compact)
                 printf(" %.2x%c%.4s   %s\n", ucp[0], (sa_v ? ',' : ' '),
                        sa_buff, name_buff);
             else
-                printf(" %.2x     %.4s       %3d    %s\n",
-                       ucp[0], sa_buff, ((ucp[6] << 8) | ucp[7]), name_buff);
+                printf(" %.2x     %.4s       %3d    %s\n", ucp[0], sa_buff,
+                       sg_get_unaligned_be16(ucp + 6), name_buff);
         if (op->do_mask) {
             int cdb_sz;
             unsigned char b[64];
@@ -608,7 +604,7 @@ list_all_codes(unsigned char * rsoc_buff, int rsoc_len, struct opts_t * op,
             res = do_rsoc(sg_fd, 0, (sa_v ? 2 : 1), opcode, serv_act,
                           b, sizeof(b), 1, op->do_verbose);
             if (0 == res) {
-                cdb_sz = (b[2] << 8) + b[3];
+                cdb_sz = sg_get_unaligned_be16(b + 2);
                 if ((cdb_sz > 0) && (cdb_sz <= 80)) {
                     if (op->do_compact)
                         printf("             usage: ");
@@ -635,13 +631,13 @@ decode_cmd_to_descriptor(unsigned char * dp, int max_b_len, char * b)
         return;
     b[max_b_len - 1] = '\0';
     --max_b_len;
-    len = (dp[0] << 8) + dp[1];
+    len = sg_get_unaligned_be16(dp + 0);
     if (10 != len) {
         snprintf(b, max_b_len, "command timeout descriptor length %d "
                  "(expect 10)", len);
         return;
     }
-    to = ((unsigned int)dp[4] << 24) + (dp[5] << 16) + (dp[6] << 8) + dp[7];
+    to = sg_get_unaligned_be32(dp + 4);
     if (0 == to)
         snprintf(b, max_b_len, "no nominal timeout, ");
     else
@@ -649,7 +645,7 @@ decode_cmd_to_descriptor(unsigned char * dp, int max_b_len, char * b)
     len = strlen(b);
     max_b_len -= len;
     b += len;
-    to = ((unsigned int)dp[8] << 24) + (dp[9] << 16) + (dp[10] << 8) + dp[11];
+    to = sg_get_unaligned_be32(dp + 8);
     if (0 == to)
         snprintf(b, max_b_len, "no recommended timeout");
     else
@@ -742,12 +738,12 @@ main(int argc, char * argv[])
         return 0;
     }
     if (op->do_version) {
-        fprintf(stderr, "Version string: %s\n", version_str);
+        pr2serr("Version string: %s\n", version_str);
         return 0;
     }
 
     if (NULL == op->device_name) {
-        fprintf(stderr, "No DEVICE argument given\n");
+        pr2serr("No DEVICE argument given\n");
         if (op->opt_new)
             usage();
         else
@@ -755,7 +751,7 @@ main(int argc, char * argv[])
         return SG_LIB_SYNTAX_ERROR;
     }
     if ((-1 != op->do_servact) && (-1 == op->do_opcode)) {
-        fprintf(stderr, "When '-s' is chosen, so must '-o' be chosen\n");
+        pr2serr("When '-s' is chosen, so must '-o' be chosen\n");
         if (op->opt_new)
             usage();
         else
@@ -763,13 +759,13 @@ main(int argc, char * argv[])
         return SG_LIB_SYNTAX_ERROR;
     }
     if (op->do_unsorted && op->do_alpha)
-        fprintf(stderr, "warning: unsorted ('-u') and alpha ('-a') options "
-                "chosen, ignoring alpha\n");
+        pr2serr("warning: unsorted ('-u') and alpha ('-a') options chosen, "
+                "ignoring alpha\n");
     if (op->do_taskman && ((-1 != op->do_opcode) || op->do_alpha ||
         op->do_unsorted)) {
-        fprintf(stderr, "warning: task management functions ('-t') chosen "
-                "so alpha ('-a'),\n          unsorted ('-u') and opcode "
-                "('-o') options ignored\n");
+        pr2serr("warning: task management functions ('-t') chosen so alpha "
+                "('-a'),\n          unsorted ('-u') and opcode ('-o') "
+                "options ignored\n");
     }
     op_name = op->do_taskman ? "Report supported task management functions" :
               "Report supported operation codes";
@@ -777,7 +773,7 @@ main(int argc, char * argv[])
     if (op->do_opcode < 0) {
         if ((sg_fd = scsi_pt_open_device(op->device_name, 1 /* RO */,
                                          op->do_verbose)) < 0) {
-            fprintf(stderr, "sg_opcodes: error opening file (ro): %s: %s\n",
+            pr2serr("sg_opcodes: error opening file (ro): %s: %s\n",
                     op->device_name, safe_strerror(-sg_fd));
             return SG_LIB_FILE_ERROR;
         }
@@ -793,20 +789,20 @@ main(int argc, char * argv[])
                     printf("  Peripheral device type: 0x%x\n", peri_type);
             }
         } else {
-            fprintf(stderr, "sg_opcodes: %s doesn't respond to a SCSI "
-                    "INQUIRY\n", op->device_name);
+            pr2serr("sg_opcodes: %s doesn't respond to a SCSI INQUIRY\n",
+                    op->device_name);
             return SG_LIB_CAT_OTHER;
         }
         res = scsi_pt_close_device(sg_fd);
         if (res < 0) {
-            fprintf(stderr, "close error: %s\n", safe_strerror(-res));
+            pr2serr("close error: %s\n", safe_strerror(-res));
             return SG_LIB_FILE_ERROR;
         }
     }
 
     if ((sg_fd = scsi_pt_open_device(op->device_name, 0 /* RW */,
                                      op->do_verbose)) < 0) {
-        fprintf(stderr, "sg_opcodes: error opening file (rw): %s: %s\n",
+        pr2serr("sg_opcodes: error opening file (rw): %s: %s\n",
                 op->device_name, safe_strerror(-sg_fd));
         return SG_LIB_FILE_ERROR;
     }
@@ -822,7 +818,7 @@ main(int argc, char * argv[])
                       op->do_verbose);
     if (res) {
         sg_get_category_sense_str(res, sizeof(b), b, op->do_verbose);
-        fprintf(stderr, "%s: %s\n", op_name, b);
+        pr2serr("%s: %s\n", op_name, b);
         goto err_out;
     }
     if (op->do_taskman) {
@@ -859,8 +855,8 @@ main(int argc, char * argv[])
             printf("    I_T nexus reset\n");
         if (op->do_repd) {
             if (rsoc_buff[3] < 0xc) {
-                fprintf(stderr, "when REPD given, byte 3 of response "
-                        "should be >= 12\n");
+                pr2serr("when REPD given, byte 3 of response should be >= "
+                        "12\n");
                 res = SG_LIB_CAT_OTHER;
                 goto err_out;
             } else
@@ -876,15 +872,12 @@ main(int argc, char * argv[])
             printf("    QTSTS=%d\n", !!(rsoc_buff[7] & 0x2));
             printf("    ITNRTS=%d\n", !!(rsoc_buff[7] & 0x1));
             printf("    tmf long timeout: %d (100 ms units)\n",
-                   (rsoc_buff[8] << 24) + (rsoc_buff[9] << 16) +
-                   (rsoc_buff[10] << 8) + rsoc_buff[11]);
+                   sg_get_unaligned_be32(rsoc_buff + 8));
             printf("    tmf short timeout: %d (100 ms units)\n",
-                   (rsoc_buff[12] << 24) + (rsoc_buff[13] << 16) +
-                   (rsoc_buff[14] << 8) + rsoc_buff[15]);
+                   sg_get_unaligned_be32(rsoc_buff + 12));
         }
     } else if (0 == rep_opts) {  /* list all supported operation codes */
-        len = ((rsoc_buff[0] << 24) | (rsoc_buff[1] << 16) |
-               (rsoc_buff[2] << 8) | rsoc_buff[3]) + 4;
+        len = sg_get_unaligned_be32(rsoc_buff + 0) + 4;
         if (len > (int)sizeof(rsoc_buff))
             len = sizeof(rsoc_buff);
         if (op->do_raw) {
@@ -897,7 +890,7 @@ main(int argc, char * argv[])
         }
         list_all_codes(rsoc_buff, sizeof(rsoc_buff), op, sg_fd);
     } else {    /* asked about specific command */
-        cd_len = ((rsoc_buff[2] << 8) | rsoc_buff[3]);
+        cd_len = sg_get_unaligned_be16(rsoc_buff + 2);
         len = cd_len + 4;
         if (len > (int)sizeof(rsoc_buff))
             len = sizeof(rsoc_buff);
@@ -934,25 +927,19 @@ do_rsoc(int sg_fd, int rctd, int rep_opts, int rq_opcode, int rq_servact,
         rsocCmdBlk[2] |= (rep_opts & 0x7);
     if (rq_opcode > 0)
         rsocCmdBlk[3] = (rq_opcode & 0xff);
-    if (rq_servact > 0) {
-        rsocCmdBlk[4] = (unsigned char)((rq_servact >> 8) & 0xff);
-        rsocCmdBlk[5] = (unsigned char)(rq_servact & 0xff);
-
-    }
-    rsocCmdBlk[6] = (unsigned char)((mx_resp_len >> 24) & 0xff);
-    rsocCmdBlk[7] = (unsigned char)((mx_resp_len >> 16) & 0xff);
-    rsocCmdBlk[8] = (unsigned char)((mx_resp_len >> 8) & 0xff);
-    rsocCmdBlk[9] = (unsigned char)(mx_resp_len & 0xff);
+    if (rq_servact > 0)
+        sg_put_unaligned_be16((uint16_t)rq_servact, rsocCmdBlk + 4);
+    sg_put_unaligned_be32((uint32_t)mx_resp_len, rsocCmdBlk + 6);
 
     if (verbose) {
-        fprintf(stderr, "    Report Supported Operation Codes cmd: ");
+        pr2serr("    Report Supported Operation Codes cmd: ");
         for (k = 0; k < RSOC_CMD_LEN; ++k)
-            fprintf(stderr, "%02x ", rsocCmdBlk[k]);
-        fprintf(stderr, "\n");
+            pr2serr("%02x ", rsocCmdBlk[k]);
+        pr2serr("\n");
     }
     ptvp = construct_scsi_pt_obj();
     if (NULL == ptvp) {
-        fprintf(stderr, "Report Supported Operation Codes: out of memory\n");
+        pr2serr("Report Supported Operation Codes: out of memory\n");
         return -1;
     }
     set_scsi_pt_cdb(ptvp, rsocCmdBlk, sizeof(rsocCmdBlk));
@@ -993,22 +980,17 @@ do_rstmf(int sg_fd, int repd, void * resp, int mx_resp_len, int noisy,
 
     if (repd)
         rstmfCmdBlk[2] = 0x80;
-    rstmfCmdBlk[6] = (unsigned char)((mx_resp_len >> 24) & 0xff);
-    rstmfCmdBlk[7] = (unsigned char)((mx_resp_len >> 16) & 0xff);
-    rstmfCmdBlk[8] = (unsigned char)((mx_resp_len >> 8) & 0xff);
-    rstmfCmdBlk[9] = (unsigned char)(mx_resp_len & 0xff);
+    sg_put_unaligned_be32((uint32_t)mx_resp_len, rstmfCmdBlk + 6);
 
     if (verbose) {
-        fprintf(stderr, "    Report Supported Task Management Functions "
-                "cmd: ");
+        pr2serr("    Report Supported Task Management Functions cmd: ");
         for (k = 0; k < RSTMF_CMD_LEN; ++k)
-            fprintf(stderr, "%02x ", rstmfCmdBlk[k]);
-        fprintf(stderr, "\n");
+            pr2serr("%02x ", rstmfCmdBlk[k]);
+        pr2serr("\n");
     }
     ptvp = construct_scsi_pt_obj();
     if (NULL == ptvp) {
-        fprintf(stderr, "Report Supported Task Management Functions: out of "
-                "memory\n");
+        pr2serr("Report Supported Task Management Functions: out of memory\n");
         return -1;
     }
     set_scsi_pt_cdb(ptvp, rstmfCmdBlk, sizeof(rstmfCmdBlk));
