@@ -16,6 +16,8 @@
 #include "sg_lib.h"
 #include "sg_cmds_basic.h"
 #include "sg_pt.h"
+#include "sg_unaligned.h"
+#include "sg_pr2serr.h"
 
 /* A utility program for the Linux OS SCSI subsystem.
  *
@@ -25,7 +27,7 @@
  * (e.g. disks).
  */
 
-static const char * version_str = "1.12 20150511";
+static const char * version_str = "1.14 20151219";
 
 #define SYNCHRONIZE_CACHE16_CMD     0x91
 #define SYNCHRONIZE_CACHE16_CMDLEN  16
@@ -49,37 +51,35 @@ static struct option long_options[] = {
 
 static void usage()
 {
-    fprintf(stderr, "Usage: "
-          "sg_sync    [--16] [--count=COUNT] [--group=GN] [--help] "
-          "[--immed]\n"
-          "                  [--lba=LBA] [--sync-nv] [--timeout=SECS] "
-          "[--verbose]\n"
-          "                  [--version] DEVICE\n"
-          "  where:\n"
-          "    --16|-S             calls SYNCHRONIZE CACHE(16) (def: is "
-          "10 byte\n"
-          "                        variant)\n"
-          "    --count=COUNT|-c COUNT    number of blocks to sync (def: 0 "
-          "which\n"
-          "                              implies rest of device)\n"
-          "    --group=GN|-g GN    set group number field to GN (def: 0)\n"
-          "    --help|-h           print out usage message\n"
-          "    --immed|-i          command returns immediately when set "
-          "else wait\n"
-          "                        for 'sync' to complete\n"
-          "    --lba=LBA|-l LBA    logical block address to start sync "
-          "operation\n"
-          "                        from (def: 0)\n"
-          "    --sync-nv|-s        synchronize to non-volatile storage "
-          "(if distinct\n"
-          "                        from medium). Obsolete in sbc3r35d.\n"
-          "    --timeout=SECS|-t SECS    command timeout in seconds, only "
-          "active\n"
-          "                              if '--16' given (def: 60 seconds)\n"
-          "    --verbose|-v        increase verbosity\n"
-          "    --version|-V        print version string and exit\n\n"
-          "Performs a SCSI SYNCHRONIZE CACHE(10 or 16) command\n"
-          );
+    pr2serr("Usage: sg_sync    [--16] [--count=COUNT] [--group=GN] [--help] "
+            "[--immed]\n"
+            "                  [--lba=LBA] [--sync-nv] [--timeout=SECS] "
+            "[--verbose]\n"
+            "                  [--version] DEVICE\n"
+            "  where:\n"
+            "    --16|-S             calls SYNCHRONIZE CACHE(16) (def: is "
+            "10 byte\n"
+            "                        variant)\n"
+            "    --count=COUNT|-c COUNT    number of blocks to sync (def: 0 "
+            "which\n"
+            "                              implies rest of device)\n"
+            "    --group=GN|-g GN    set group number field to GN (def: 0)\n"
+            "    --help|-h           print out usage message\n"
+            "    --immed|-i          command returns immediately when set "
+            "else wait\n"
+            "                        for 'sync' to complete\n"
+            "    --lba=LBA|-l LBA    logical block address to start sync "
+            "operation\n"
+            "                        from (def: 0)\n"
+            "    --sync-nv|-s        synchronize to non-volatile storage "
+            "(if distinct\n"
+            "                        from medium). Obsolete in sbc3r35d.\n"
+            "    --timeout=SECS|-t SECS    command timeout in seconds, only "
+            "active\n"
+            "                              if '--16' given (def: 60 seconds)\n"
+            "    --verbose|-v        increase verbosity\n"
+            "    --version|-V        print version string and exit\n\n"
+            "Performs a SCSI SYNCHRONIZE CACHE(10 or 16) command\n");
 }
 
 static int
@@ -98,29 +98,19 @@ ll_sync_cache_16(int sg_fd, int sync_nv, int immed, int group,
         scCmdBlk[1] |= 4;       /* obsolete in sbc3r35d */
     if (immed)
         scCmdBlk[1] |= 2;
-    scCmdBlk[2] = (lba >> 56) & 0xff;
-    scCmdBlk[3] = (lba >> 48) & 0xff;
-    scCmdBlk[4] = (lba >> 40) & 0xff;
-    scCmdBlk[5] = (lba >> 32) & 0xff;
-    scCmdBlk[6] = (lba >> 24) & 0xff;
-    scCmdBlk[7] = (lba >> 16) & 0xff;
-    scCmdBlk[8] = (lba >> 8) & 0xff;
-    scCmdBlk[9] = lba & 0xff;
+    sg_put_unaligned_be64(lba, scCmdBlk + 2);
     scCmdBlk[14] = group & 0x1f;
-    scCmdBlk[10] = (num_lb >> 24) & 0xff;
-    scCmdBlk[11] = (num_lb >> 16) & 0xff;
-    scCmdBlk[12] = (num_lb >> 8) & 0xff;
-    scCmdBlk[13] = num_lb & 0xff;
+    sg_put_unaligned_be32((uint32_t)num_lb, scCmdBlk + 10);
 
     if (verbose) {
-        fprintf(stderr, "    synchronize cache(16) cdb: ");
+        pr2serr("    synchronize cache(16) cdb: ");
         for (k = 0; k < SYNCHRONIZE_CACHE16_CMDLEN; ++k)
-            fprintf(stderr, "%02x ", scCmdBlk[k]);
-        fprintf(stderr, "\n");
+            pr2serr("%02x ", scCmdBlk[k]);
+        pr2serr("\n");
     }
     ptvp = construct_scsi_pt_obj();
     if (NULL == ptvp) {
-        fprintf(stderr, "synchronize cache(16): out of memory\n");
+        pr2serr("synchronize cache(16): out of memory\n");
         return -1;
     }
     set_scsi_pt_cdb(ptvp, scCmdBlk, sizeof(scCmdBlk));
@@ -175,7 +165,7 @@ int main(int argc, char * argv[])
         case 'c':
             count = sg_get_llnum(optarg);
             if ((count < 0) || (count > UINT_MAX)) {
-                fprintf(stderr, "bad argument to '--count'\n");
+                pr2serr("bad argument to '--count'\n");
                 return SG_LIB_SYNTAX_ERROR;
             }
             num_lb = (unsigned int)count;
@@ -183,7 +173,7 @@ int main(int argc, char * argv[])
         case 'g':
             group = sg_get_num(optarg);
             if ((group < 0) || (group > 31)) {
-                fprintf(stderr, "bad argument to '--group'\n");
+                pr2serr("bad argument to '--group'\n");
                 return SG_LIB_SYNTAX_ERROR;
             }
             break;
@@ -197,7 +187,7 @@ int main(int argc, char * argv[])
         case 'l':
             lba = sg_get_llnum(optarg);
             if (lba < 0) {
-                fprintf(stderr, "bad argument to '--lba'\n");
+                pr2serr("bad argument to '--lba'\n");
                 return SG_LIB_SYNTAX_ERROR;
             }
             break;
@@ -210,7 +200,7 @@ int main(int argc, char * argv[])
         case 't':
             to_secs = sg_get_num(optarg);
             if (to_secs < 0) {
-                fprintf(stderr, "bad argument to '--timeout'\n");
+                pr2serr("bad argument to '--timeout'\n");
                 return SG_LIB_SYNTAX_ERROR;
             }
             break;
@@ -218,10 +208,10 @@ int main(int argc, char * argv[])
             ++verbose;
             break;
         case 'V':
-            fprintf(stderr, "version: %s\n", version_str);
+            pr2serr("version: %s\n", version_str);
             return 0;
         default:
-            fprintf(stderr, "unrecognised option code 0x%x ??\n", c);
+            pr2serr("unrecognised option code 0x%x ??\n", c);
             usage();
             return SG_LIB_SYNTAX_ERROR;
         }
@@ -233,21 +223,20 @@ int main(int argc, char * argv[])
         }
         if (optind < argc) {
             for (; optind < argc; ++optind)
-                fprintf(stderr, "Unexpected extra argument: %s\n",
-                        argv[optind]);
+                pr2serr("Unexpected extra argument: %s\n", argv[optind]);
             usage();
             return SG_LIB_SYNTAX_ERROR;
         }
     }
 
     if (NULL == device_name) {
-        fprintf(stderr, "missing device name!\n");
+        pr2serr("missing device name!\n");
         usage();
         return SG_LIB_SYNTAX_ERROR;
     }
     sg_fd = sg_cmds_open_device(device_name, 0 /* rw */, verbose);
     if (sg_fd < 0) {
-        fprintf(stderr, "open error: %s: %s\n", device_name,
+        pr2serr("open error: %s: %s\n", device_name,
                 safe_strerror(-sg_fd));
         return SG_LIB_FILE_ERROR;
     }
@@ -263,12 +252,12 @@ int main(int argc, char * argv[])
         char b[80];
 
         sg_get_category_sense_str(res, sizeof(b), b, verbose);
-        fprintf(stderr, "Synchronize cache failed: %s\n", b);
+        pr2serr("Synchronize cache failed: %s\n", b);
     }
 
     res = sg_cmds_close_device(sg_fd);
     if (res < 0) {
-        fprintf(stderr, "close error: %s\n", safe_strerror(-res));
+        pr2serr("close error: %s\n", safe_strerror(-res));
         if (0 == ret)
             return SG_LIB_FILE_ERROR;
     }
